@@ -9,19 +9,22 @@ import (
 	"reflect"
 	"testing"
 
+	configv1 "github.com/openshift/api/config/v1"
 	imageregistryv1 "github.com/openshift/api/imageregistry/v1"
 	networkv1 "github.com/openshift/api/network/v1"
+	configfake "github.com/openshift/client-go/config/clientset/versioned/fake"
 	corev1 "k8s.io/api/core/v1"
+	policyv1beta1 "k8s.io/api/policy/v1beta1"
 	"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1"
 	apixv1beta1clientfake "k8s.io/apiextensions-apiserver/pkg/client/clientset/clientset/fake"
-	policyv1beta1 "k8s.io/api/policy/v1beta1"
+	"k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/runtime/schema"
 	"k8s.io/apimachinery/pkg/runtime/serializer/yaml"
-	dynamicfake "k8s.io/client-go/dynamic/fake"
 	"k8s.io/apimachinery/pkg/util/intstr"
+	dynamicfake "k8s.io/client-go/dynamic/fake"
 	kubefake "k8s.io/client-go/kubernetes/fake"
 	"k8s.io/klog"
 
@@ -158,7 +161,7 @@ func TestGatherClusterPruner(t *testing.T) {
 			if test.expectedRecords == 0 {
 				return
 			}
-			if expectedRecordName := "config/imagepruner"; records[0].Name != expectedRecordName {
+			if expectedRecordName := "config/clusteroperator/imageregistry.operator.openshift.io/imagepruner/cluster"; records[0].Name != expectedRecordName {
 				t.Errorf("expected %q record name, got %q", expectedRecordName, records[0].Name)
 				return
 			}
@@ -177,18 +180,18 @@ func TestGatherClusterPruner(t *testing.T) {
 	}
 }
 
-func TestGatherPodDisruptionBudgets(t *testing.T){
+func TestGatherPodDisruptionBudgets(t *testing.T) {
 	coreClient := kubefake.NewSimpleClientset()
 
 	fakeNamespace := "fake-namespace"
 
 	// name -> MinAvailabel
-	fakePDBs := map[string]string {
-		"pdb-four": "4",
+	fakePDBs := map[string]string{
+		"pdb-four":  "4",
 		"pdb-eight": "8",
-		"pdb-ten": "10",
+		"pdb-ten":   "10",
 	}
-	for name, minAvailable := range fakePDBs{
+	for name, minAvailable := range fakePDBs {
 		_, err := coreClient.PolicyV1beta1().
 			PodDisruptionBudgets(fakeNamespace).
 			Create(context.Background(), &policyv1beta1.PodDisruptionBudget{
@@ -222,7 +225,7 @@ func TestGatherPodDisruptionBudgets(t *testing.T){
 		}
 		name := pdba.PodDisruptionBudget.ObjectMeta.Name
 		minAvailable := pdba.PodDisruptionBudget.Spec.MinAvailable.StrVal
-		if pdba.PodDisruptionBudget.Spec.MinAvailable.StrVal !=  fakePDBs[name] {
+		if pdba.PodDisruptionBudget.Spec.MinAvailable.StrVal != fakePDBs[name] {
 			t.Fatalf("pdb item has mismatched MinAvailable value, %q != %q", fakePDBs[name], minAvailable)
 		}
 	}
@@ -349,7 +352,7 @@ func TestGatherClusterImageRegistry(t *testing.T) {
 				t.Errorf("expected one record, got %d", numRecords)
 				return
 			}
-			if expectedRecordName := "config/imageregistry"; records[0].Name != expectedRecordName {
+			if expectedRecordName := "config/clusteroperator/imageregistry.operator.openshift.io/config/cluster"; records[0].Name != expectedRecordName {
 				t.Errorf("expected %q record name, got %q", expectedRecordName, records[0].Name)
 				return
 			}
@@ -378,18 +381,12 @@ func TestGatherContainerImages(t *testing.T) {
 		"registry.redhat.io/3",
 	}
 
-	expected := ContainerInfo{
-		Images: ContainerImageSet{
-			0: "registry.redhat.io/1",
-			1: "registry.redhat.io/2",
-			2: "registry.redhat.io/3",
-		},
-		Containers: PodsWithAge{
-			"0001-01": RunningImages{
-				0: 1,
-				1: 1,
-				2: 1,
-			},
+	// It is not possible to predict the order of the images.
+	expectedPodsWithAge := PodsWithAge{
+		"0001-01": RunningImages{
+			0: 1,
+			1: 1,
+			2: 1,
 		},
 	}
 
@@ -402,15 +399,13 @@ func TestGatherContainerImages(t *testing.T) {
 					Namespace: fakeNamespace,
 					Name:      fmt.Sprintf("pod%d", index),
 				},
-				Spec: corev1.PodSpec{
-					Containers: []corev1.Container{
+				Status: corev1.PodStatus{
+					ContainerStatuses: []corev1.ContainerStatus{
 						{
 							Name:  fmt.Sprintf("container%d", index),
 							Image: containerImage,
 						},
 					},
-				},
-				Status: corev1.PodStatus{
 					Phase: corev1.PodRunning,
 				},
 			}, metav1.CreateOptions{})
@@ -448,8 +443,7 @@ func TestGatherContainerImages(t *testing.T) {
 		expectedRecords[i] = fmt.Sprintf("config/pod/%s/%s", fakeOpenshiftNamespace, podName)
 	}
 
-	gatherer := &Gatherer{coreClient: coreClient.CoreV1()}
-
+	gatherer := &Gatherer{ctx: context.Background(), coreClient: coreClient.CoreV1()}
 	records, errs := GatherContainerImages(gatherer)()
 	if len(errs) > 0 {
 		t.Errorf("unexpected errors: %#v", errs)
@@ -477,8 +471,12 @@ func TestGatherContainerImages(t *testing.T) {
 		t.Fatal("container info has not been reported")
 	}
 
-	if !reflect.DeepEqual(*containerInfo, expected) {
-		t.Fatalf("unexpected result: %#v", *containerInfo)
+	if len(containerInfo.Images) != len(mockContainers) {
+		t.Fatalf("expected %d unique images, got %d", len(mockContainers), len(containerInfo.Images))
+	}
+
+	if !reflect.DeepEqual(containerInfo.Containers, expectedPodsWithAge) {
+		t.Fatalf("unexpected map of image counts: %#v", containerInfo.Containers)
 	}
 
 	for _, expectedRecordName := range expectedRecords {
@@ -629,6 +627,171 @@ metadata:
 	}
 }
 
+func TestGatherClusterOperator(t *testing.T) {
+	testOperator := &configv1.ClusterOperator{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: "test-clusteroperator",
+		},
+	}
+	configCS := configfake.NewSimpleClientset()
+	_, err := configCS.ConfigV1().ClusterOperators().Create(context.Background(), testOperator, metav1.CreateOptions{})
+	if err != nil {
+		t.Fatal("unable to create fake clusteroperator", err)
+	}
+	gatherer := &Gatherer{client: configCS.ConfigV1(), discoveryClient: configCS.Discovery()}
+	records, errs := GatherClusterOperators(gatherer)()
+	if len(errs) > 0 {
+		t.Errorf("unexpected errors: %#v", errs)
+		return
+	}
+
+	item, _ := records[0].Item.Marshal(context.TODO())
+	var gatheredCO configv1.ClusterOperator
+	_, _, err = openshiftSerializer.Decode(item, nil, &gatheredCO)
+	if err != nil {
+		t.Fatalf("failed to decode object: %v", err)
+	}
+	if gatheredCO.Name != "test-clusteroperator" {
+		t.Fatalf("unexpected clusteroperator name %s", gatheredCO.Name)
+	}
+}
+func TestGatherMachineConfigPool(t *testing.T) {
+	var machineconfigpoolYAML = `
+apiVersion: machineconfiguration.openshift.io/v1
+kind: MachineConfigPool
+metadata:
+    name: master-t
+`
+	gvr := schema.GroupVersionResource{Group: "machineconfiguration.openshift.io", Version: "v1", Resource: "machineconfigpools"}
+	client := dynamicfake.NewSimpleDynamicClient(runtime.NewScheme())
+	decUnstructured := yaml.NewDecodingSerializer(unstructured.UnstructuredJSONScheme)
+
+	testMachineConfigPools := &unstructured.Unstructured{}
+
+	_, _, err := decUnstructured.Decode([]byte(machineconfigpoolYAML), nil, testMachineConfigPools)
+	if err != nil {
+		t.Fatal("unable to decode machineconfigpool ", err)
+	}
+	_, err = client.Resource(gvr).Create(context.Background(), testMachineConfigPools, metav1.CreateOptions{})
+	if err != nil {
+		t.Fatal("unable to create fake machineconfigpool ", err)
+	}
+
+	gatherer := &Gatherer{dynamicClient: client}
+	records, errs := GatherMachineConfigPool(gatherer)()
+	if len(errs) > 0 {
+		t.Errorf("unexpected errors: %#v", errs)
+		return
+	}
+	if len(records) != 1 {
+		t.Fatalf("unexpected number or records %d", len(records))
+	}
+	if records[0].Name != "config/machineconfigpools/master-t" {
+		t.Fatalf("unexpected machineconfigpool name %s", records[0].Name)
+	}
+}
+
+func TestGatherInstallPlans(t *testing.T) {
+	tests := []struct {
+		name      string
+		testfiles []string
+		limit     int
+		exp       string
+	}{
+		{
+			name:      "one installplan",
+			testfiles: []string{"testdata/installplan.yaml"},
+			exp: `{"items":[{"count":1,"csv":"lib-bucket-provisioner.v2.0.0","name":"install-","ns":"openshift-operators"}],` +
+				`"stats":{"TOTAL_COUNT":1,"TOTAL_NONUNIQ_COUNT":1}}`,
+		},
+		{
+			name:      "two are same to keep ordering and one is different",
+			testfiles: []string{"testdata/installplan.yaml", "testdata/installplan2.yaml", "testdata/installplan_openshift.yaml"},
+			exp:       `{"items":[{"count":2,"csv":"lib-bucket-provisioner.v2.0.0","name":"install-","ns":"openshift-operators"},{"count":1,"csv":"3scale-community-operator.v0.5.1","name":"install-","ns":"openshift"}],"stats":{"TOTAL_COUNT":3,"TOTAL_NONUNIQ_COUNT":2}}`,
+		},
+		{
+			name:      "two similar installplans",
+			testfiles: []string{"testdata/installplan.yaml", "testdata/installplan2.yaml"},
+			exp: `{"items":[{"count":2,"csv":"lib-bucket-provisioner.v2.0.0","name":"install-","ns":"openshift-operators"}],` +
+				`"stats":{"TOTAL_COUNT":2,"TOTAL_NONUNIQ_COUNT":1}}`,
+		},
+		{
+			name:      "test marshaller with limit to 1 item",
+			testfiles: []string{"testdata/installplan.yaml", "testdata/installplan2.yaml", "testdata/installplan_openshift.yaml"},
+			limit:     1,
+			exp:       `{"items":[{"count":2,"csv":"lib-bucket-provisioner.v2.0.0","name":"install-","ns":"openshift-operators"}],"stats":{"TOTAL_COUNT":3,"TOTAL_NONUNIQ_COUNT":2}}`,
+		},
+	}
+
+	for _, test := range tests {
+		t.Run(test.name, func(t *testing.T) {
+
+			client := dynamicfake.NewSimpleDynamicClient(runtime.NewScheme())
+			coreClient := kubefake.NewSimpleClientset()
+			for _, file := range test.testfiles {
+				f, err := os.Open(file)
+				if err != nil {
+					t.Fatal("test failed to read installplan data", err)
+				}
+				defer f.Close()
+				installplancontent, err := ioutil.ReadAll(f)
+				if err != nil {
+					t.Fatal("error reading test data file", err)
+				}
+
+				decUnstructured := yaml.NewDecodingSerializer(unstructured.UnstructuredJSONScheme)
+				installplan := &unstructured.Unstructured{}
+
+				_, _, err = decUnstructured.Decode(installplancontent, nil, installplan)
+				if err != nil {
+					t.Fatal("unable to decode", err)
+				}
+				gv, _ := schema.ParseGroupVersion(installplan.GetAPIVersion())
+				gvr := schema.GroupVersionResource{Version: gv.Version, Group: gv.Group, Resource: "installplans"}
+				var ns string
+				err = parseJSONQuery(installplan.Object, "metadata.namespace", &ns)
+				if err != nil {
+					t.Fatal("unable to read ns ", err)
+				}
+				_, err = coreClient.CoreV1().Namespaces().Get(context.Background(), ns, metav1.GetOptions{})
+				if errors.IsNotFound(err) {
+					_, err = coreClient.CoreV1().Namespaces().Create(context.Background(), &corev1.Namespace{ObjectMeta: metav1.ObjectMeta{Name: ns}}, metav1.CreateOptions{})
+				}
+				if err != nil {
+					t.Fatal("unable to create ns fake ", err)
+				}
+				_, err = client.Resource(gvr).Namespace(ns).Create(context.Background(), installplan, metav1.CreateOptions{})
+				if err != nil {
+					t.Fatal("unable to create installplan fake ", err)
+				}
+			}
+
+			gatherer := &Gatherer{ctx: context.Background(), dynamicClient: client, coreClient: coreClient.CoreV1()}
+			records, errs := GatherInstallPlans(gatherer)()
+			if len(errs) > 0 {
+				t.Errorf("unexpected errors: %#v", errs)
+				return
+			}
+			if len(records) != 1 {
+				t.Fatalf("unexpected number or records %d", len(records))
+			}
+			m, ok := records[0].Item.(InstallPlanAnonymizer)
+			if !ok {
+				t.Fatalf("returned item is not of type InstallPlanAnonymizer")
+			}
+			if test.limit != 0 {
+				// copy to new anonymizer with limited max
+				m = InstallPlanAnonymizer{limit: 1, total: m.total, v: m.v}
+			}
+			b, _ := m.Marshal(context.Background())
+			sb := string(b)
+			if sb != test.exp {
+				t.Fatalf("unexpected installplan exp: %s got: %s", test.exp, sb)
+			}
+		})
+	}
+}
+
 func ExampleGatherMostRecentMetrics_Test() {
 	b, err := ExampleMostRecentMetrics()
 	if err != nil {
