@@ -199,7 +199,7 @@ func TestArchiveContains(t *testing.T) {
 	t.Run("ImageRegistry",
 		genLatestArchiveCheckPattern(
 			"image registry", matchingFileExists,
-			`^config/imageregistry\.json$`))
+			`^config/clusteroperator/imageregistry.operator.openshift.io/config/cluster\.json$`))
 
 	//https://bugzilla.redhat.com/show_bug.cgi?id=1873101
 	t.Run("SnapshotsCRD",
